//
//  ViewController.swift
//  BT-Centrals
//
//  Created by Lenny on 2018/3/8.
//  Copyright © 2018年 Lenny. All rights reserved.
//

import UIKit
import CoreBluetooth

class ViewController: UIViewController {

    /* 請參照文件給予對應的UUID
     Service -
     https://www.bluetooth.com/specifications/gatt/services
     
     Characteristic-
     https://www.bluetooth.com/specifications/gatt/characteristics
     */
    private let Service_UUID: String = "CDD1"
    private let Characteristic_UUID: String = "CDD2"

    @IBOutlet weak var textField: UITextField!
    private var centralManager: CBCentralManager?
    private var peripheral: CBPeripheral?
    private var characteristic: CBCharacteristic?
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        title = "藍牙中心裝備"
        centralManager = CBCentralManager.init(delegate: self, queue: .main)
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    
    @IBAction func didClickGetData(_ sender: Any) {
        self.peripheral?.readValue(for: self.characteristic!)
    }
    
    @IBAction func didClickSendData(_ sender: Any) {
        let data = (self.textField.text ?? "empty input")!.data(using: String.Encoding.utf8)
        self.peripheral?.writeValue(data!, for: self.characteristic!, type: CBCharacteristicWriteType.withResponse)
    }
    
    
}



extension ViewController: CBCentralManagerDelegate, CBPeripheralDelegate {
    
    // 判断手机蓝牙状态
    func centralManagerDidUpdateState(_ central: CBCentralManager) {
        switch central.state {
        case .unknown:
            print("未知的")
        case .resetting:
            print("重置中")
        case .unsupported:
            print("未支持")
        case .unauthorized:
            print("未驗證")
        case .poweredOff:
            print("關閉")
        case .poweredOn:
            print("啟動")
            
            central.scanForPeripherals(withServices: [CBUUID.init(string: Service_UUID)], options: nil)
        }
    }
    
    /** 找到符合要求的外部設備 */
    func centralManager(_ central: CBCentralManager, didDiscover peripheral: CBPeripheral, advertisementData: [String : Any], rssi RSSI: NSNumber) {
        self.peripheral = peripheral

        central.connect(peripheral, options: nil)
    }
    
    /** didConnect Success */
    func centralManager(_ central: CBCentralManager, didConnect peripheral: CBPeripheral) {
        self.centralManager?.stopScan()
        peripheral.delegate = self
        peripheral.discoverServices([CBUUID.init(string: Service_UUID)])
        print("didConnect Success")
    }
    
    func centralManager(_ central: CBCentralManager, didFailToConnect peripheral: CBPeripheral, error: Error?) {
        print("didFailToConnect")
    }
    
    func centralManager(_ central: CBCentralManager, didDisconnectPeripheral peripheral: CBPeripheral, error: Error?) {
        print("didDisconnectPeripheral")
        // Reconnect
        central.connect(peripheral, options: nil)
    }
    
    /** Find Service */
    func peripheral(_ peripheral: CBPeripheral, didDiscoverServices error: Error?) {
        for service: CBService in peripheral.services! {
            print("外部設備中的服務有：\(service)")
        }
      
        let service = peripheral.services?.last
        peripheral.discoverCharacteristics([CBUUID.init(string: Characteristic_UUID)], for: service!)
    }
    
    /** Find Characteristic */
    func peripheral(_ peripheral: CBPeripheral, didDiscoverCharacteristicsFor service: CBService, error: Error?) {
        for characteristic: CBCharacteristic in service.characteristics! {
            print("外部設備中的特徵有：\(characteristic)")
        }
        
        self.characteristic = service.characteristics?.last
        // Read characteristic
        peripheral.readValue(for: self.characteristic!)
        // 訂閱
        peripheral.setNotifyValue(true, for: self.characteristic!)
    }
    

    func peripheral(_ peripheral: CBPeripheral, didUpdateNotificationStateFor characteristic: CBCharacteristic, error: Error?) {
        if let error = error {
            print("訂閱失敗: \(error)")
            return
        }
        if characteristic.isNotifying {
            print("訂閱成功")
        } else {
            print("取消訂閱")
        }
    }
    
    // 外部設備發送數據時， 接收數據並寫入textField
    func peripheral(_ peripheral: CBPeripheral, didUpdateValueFor characteristic: CBCharacteristic, error: Error?) {
        let data = characteristic.value
        self.textField.text = String.init(data: data!, encoding: String.Encoding.utf8)
    }
    
    /** write data */
    func peripheral(_ peripheral: CBPeripheral, didWriteValueFor characteristic: CBCharacteristic, error: Error?) {
        print("write data")
    }
}




