//
//  ViewController.swift
//  BT-Peripherals
//
//  Created by Lenny on 2018/3/8.
//  Copyright © 2018年 Lenny. All rights reserved.
//

import UIKit
import CoreBluetooth

private let Service_UUID: String = "CDD1"
private let Characteristic_UUID: String = "CDD2"

/* 請參照文件給予對應的UUID
 Service -
 https://www.bluetooth.com/specifications/gatt/services
 
 Characteristic-
 https://www.bluetooth.com/specifications/gatt/characteristics
 */

class ViewController: UIViewController {

    @IBOutlet weak var textField: UITextField!
    
    private var peripheralManager: CBPeripheralManager?
    private var characteristic: CBMutableCharacteristic?

    
    override func viewDidLoad() {
        super.viewDidLoad()

        title = "藍牙外部設備"
        peripheralManager = CBPeripheralManager.init(delegate: self, queue: .main)

    }

    @IBAction func didClickBtnAndSendData(_ sender: Any) {
        peripheralManager?.updateValue((textField.text ?? "empty data!").data(using: .utf8)!, for: characteristic!, onSubscribedCentrals: nil)
    }
    
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }


}

extension ViewController: CBPeripheralManagerDelegate {
    func peripheralManagerDidUpdateState(_ peripheral: CBPeripheralManager) {
        switch peripheral.state {
        
        case .unknown:
            print("未知的")
        case .resetting:
            print("重置中")
        case .unsupported:
            print("未支持")
        case .unauthorized:
            print("未驗證")
        case .poweredOff:
            print("關閉")
        case .poweredOn:
            print("啟動")
            // 在此創建服務 、 廣播
            setupServiceAndCharacteristics()
        
            self.peripheralManager?.startAdvertising([CBAdvertisementDataServiceUUIDsKey : [CBUUID.init(string: Service_UUID)]])
        }
    }
    private func setupServiceAndCharacteristics() {
        let serviceID = CBUUID.init(string: Service_UUID)
        let service = CBMutableService.init(type: serviceID, primary: true)
        let characteristicID = CBUUID.init(string: Characteristic_UUID)
        let characteristic = CBMutableCharacteristic.init(type: characteristicID,
                                                          properties: [.read, .write, .notify],
                                                          value: nil,
                                                          permissions: [.readable, .writeable])
        service.characteristics = [characteristic]
        self.peripheralManager?.add(service)
        self.characteristic = characteristic
    }
    
    // 當中心設備讀取數據時的callback
    func peripheralManager(_ peripheral: CBPeripheralManager, didReceiveRead request: CBATTRequest) {
        // 把textfield中的文字發給中心設備
        request.value = self.textField.text?.data(using: .utf8)
        // 成功响应请求
        peripheral.respond(to: request, withResult: .success)
    }
    
    // 中心設備發送數據時， 接收數據並寫入textField
    func peripheralManager(_ peripheral: CBPeripheralManager, didReceiveWrite requests: [CBATTRequest]) {
        let request = requests.last!
        self.textField.text = String.init(data: request.value!, encoding: String.Encoding.utf8)
    }
    
    
    /** Subscribe characteristic success */
    func peripheralManager(_ peripheral: CBPeripheralManager, central: CBCentral, didSubscribeTo characteristic: CBCharacteristic) {
        print("\(#function) Subscribe characteristic success")
    }
    
    /** Unsubscribe characteristic success */
    func peripheralManager(_ peripheral: CBPeripheralManager, central: CBCentral, didUnsubscribeFrom characteristic: CBCharacteristic) {
        print("\(#function) Unsubscribe characteristic success")
    }
    
    
}
